"use strict";
/*
 * Copyright (c) 2020, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const path = require("path");
const i18n_1 = require("../i18n");
const index_1 = require("./index");
const index_2 = require("../metadata-registry/index");
// tslint:disable-next-line:no-var-requires
const DOMParser = require('xmldom-sfdx-encoding').DOMParser;
const CONTAINER_ASYNC_REQUEST = 'ContainerAsyncRequest';
const METADATA_CONTAINER = 'MetadataContainer';
class Deploy {
    constructor(connection, apiVersion, registryAccess) {
        this.connection = connection;
        this.apiVersion = apiVersion;
        if (registryAccess) {
            this.registryAccess = registryAccess;
        }
        else {
            this.registryAccess = new index_2.RegistryAccess();
        }
    }
    deploy(filePath) {
        return __awaiter(this, void 0, void 0, function* () {
            const component = this.registryAccess.getComponentsFromPath(filePath)[0];
            this.metadataType = component.type.name;
            const sourcePath = component.sources[0];
            const metadataPath = component.xml;
            if (index_1.supportedToolingTypes.get(this.metadataType) === undefined) {
                const deployFailed = new Error();
                deployFailed.message = i18n_1.nls.localize('beta_tapi_membertype_unsupported_error', this.metadataType);
                deployFailed.name = 'MetadataTypeUnsupported';
                throw deployFailed;
            }
            const container = yield this.createMetadataContainer();
            yield this.createContainerMember([sourcePath, metadataPath], container);
            const asyncRequest = yield this.createContainerAsyncRequest(container);
            const output = yield this.toolingStatusCheck(asyncRequest);
            return output;
        });
    }
    createMetadataContainer() {
        return __awaiter(this, void 0, void 0, function* () {
            const metadataContainer = yield this.toolingCreate(METADATA_CONTAINER, {
                Name: `VSCode_MDC_${Date.now()}`
            });
            if (!metadataContainer.success) {
                const deployFailed = new Error();
                deployFailed.message = i18n_1.nls.localize('beta_tapi_mdcontainer_error');
                deployFailed.name = 'MetadataContainerCreationFailed';
                throw deployFailed;
            }
            return metadataContainer;
        });
    }
    toolingCreate(type, record) {
        return __awaiter(this, void 0, void 0, function* () {
            return (yield this.connection.tooling.create(type, record));
        });
    }
    buildMetadataField(metadataContent) {
        const parser = new DOMParser();
        const document = parser.parseFromString(metadataContent, 'text/xml');
        const apiVersion = this.apiVersion ||
            document.getElementsByTagName('apiVersion')[0].textContent;
        const statusNode = document.getElementsByTagName('status')[0];
        const packageNode = document.getElementsByTagName('packageVersions')[0];
        const labelNode = document.getElementsByTagName('label')[0];
        const metadataField = Object.assign({ apiVersion }, (statusNode ? { status: statusNode.textContent } : {}), (packageNode ? { packageVersions: packageNode.textContent } : {}), (labelNode ? { label: labelNode.textContent } : {}));
        return metadataField;
    }
    createContainerMember(outboundFiles, container) {
        return __awaiter(this, void 0, void 0, function* () {
            const id = container.id;
            const metadataContent = fs.readFileSync(outboundFiles[1], 'utf8');
            const metadataField = this.buildMetadataField(metadataContent);
            const body = fs.readFileSync(outboundFiles[0], 'utf8');
            const fileName = path.basename(outboundFiles[0], path.extname(outboundFiles[0]));
            const contentEntity = yield this.getContentEntity(this.metadataType, fileName);
            const containerMemberObject = Object.assign({ MetadataContainerId: id, FullName: fileName, Body: body, Metadata: metadataField }, (contentEntity ? { contentEntityId: contentEntity.Id } : {}));
            const containerMember = yield this.toolingCreate(index_1.supportedToolingTypes.get(this.metadataType), containerMemberObject);
            if (!containerMember.success) {
                const deployFailed = new Error();
                deployFailed.message = i18n_1.nls.localize('beta_tapi_membertype_error', this.metadataType);
                deployFailed.name = this.metadataType.concat('MemberCreationFailed');
                throw deployFailed;
            }
            return containerMember;
        });
    }
    getContentEntity(metadataType, fileName) {
        return __awaiter(this, void 0, void 0, function* () {
            return (yield this.connection.tooling
                .sobject(metadataType)
                .find({ Name: fileName }))[0];
        });
    }
    createContainerAsyncRequest(container) {
        return __awaiter(this, void 0, void 0, function* () {
            const contAsyncRequest = yield this.toolingCreate(CONTAINER_ASYNC_REQUEST, {
                MetadataContainerId: container.id
            });
            if (!contAsyncRequest.success) {
                const deployFailed = new Error();
                deployFailed.message = i18n_1.nls.localize('beta_tapi_car_error');
                deployFailed.name = 'ContainerAsyncRequestFailed';
                throw deployFailed;
            }
            return contAsyncRequest;
        });
    }
    sleep(ms) {
        return new Promise(resolve => setTimeout(resolve, ms));
    }
    toolingStatusCheck(asyncRequest) {
        return __awaiter(this, void 0, void 0, function* () {
            let retrieveResult = yield this.toolingRetrieve(CONTAINER_ASYNC_REQUEST, asyncRequest.id);
            let count = 0;
            while (retrieveResult.State === index_1.DeployStatusEnum.Queued && count <= 30) {
                yield this.sleep(100);
                retrieveResult = (yield this.toolingRetrieve(CONTAINER_ASYNC_REQUEST, asyncRequest.id));
                count++;
            }
            return retrieveResult;
        });
    }
    toolingRetrieve(type, id) {
        return __awaiter(this, void 0, void 0, function* () {
            return (yield this.connection.tooling.retrieve(type, id));
        });
    }
}
exports.Deploy = Deploy;
//# sourceMappingURL=toolingDeploy.js.map